﻿using System;
using System.Collections.Generic;
using IMSES_Template.Model;

namespace IMSES_Template.DataAccess
{
    public interface IClient : IDisposable
    {
        #region User

        OperationResult CreateUser(User newUser);
        OperationResult ReadUser(string syncKey, out User readUser);
        OperationResult UpdateUser(User updatedUser);
        OperationResult ReplaceUser(User updatedUser);
        OperationResult DeleteUser(string syncKey);

        BatchOperationResult CreateUsers(ICollection<User> newUsers);
        BatchOperationResult ReadUsers(ICollection<string> syncKeys, out List<User> readUserList);
        BatchOperationResult UpdateUsers(ICollection<User> updatedUsers);
        BatchOperationResult ReplaceUsers(ICollection<User> updatedUsers);
        BatchOperationResult DeleteUsers(ICollection<string> syncKeys);

        OperationResult ChangeUserKey(string oldKey, string newKey);
        BatchOperationResult ChangeUserKeys(ICollection<KeyPair> syncKeyPairs);

        OperationResult ReadUsersFromHierarchy(string syncId, out List<User> readUserList);
        OperationResult ReadAllUsers(out List<User> readUserList);

        #endregion

        #region Hierarchy

        OperationResult CreateHierarchy(Hierarchy newHierarchy);
        OperationResult ReadHierarchy(string syncId, out Hierarchy readHierarchy);
        OperationResult UpdateHierarchy(Hierarchy updatedHierarchy);
        OperationResult ReplaceHierarchy(Hierarchy updatedHierarchy);
        OperationResult DeleteHierarchy(string syncId);

        BatchOperationResult CreateHierarchies(ICollection<Hierarchy> newHierarchies);
        BatchOperationResult ReadHierarchies(ICollection<string> syncIds, out List<Hierarchy> readHierarchyList);
        BatchOperationResult UpdateHierarchies(ICollection<Hierarchy> updatedHierarchies);
        BatchOperationResult ReplaceHierarchies(ICollection<Hierarchy> updatedHierarchies);
        BatchOperationResult DeleteHierarchies(ICollection<string> syncIds);

        OperationResult ChangeHierarchyId(string oldId, string newId);
        BatchOperationResult ChangeHierarchyIds(ICollection<KeyPair> syncIdPairs);

        OperationResult ReadHierarchiesFromUser(string syncKey, out List<Hierarchy> readHierarchyList);

        #endregion

        #region Course

        OperationResult CreateCourse(Course newCourse);
        OperationResult ReadCourse(string syncId, out Course readCourse);
        OperationResult UpdateCourse(Course updatedCourse);
        OperationResult ReplaceCourse(Course updatedCourse);
        // DeleteCourse does not exists, you can only delete the linked Hierarchy, never the Course itself.

        BatchOperationResult CreateCourses(ICollection<Course> newCourses);
        BatchOperationResult ReadCourses(ICollection<string> syncIds, out List<Course> readCourseList);
        BatchOperationResult UpdateCourses(ICollection<Course> updatedCourses);
        BatchOperationResult ReplaceCourses(ICollection<Course> updatedCourses);

        OperationResult ReadCoursesFromUser(string syncKey, out List<Course> readCourseList);

        #endregion

        #region CourseGroup

        OperationResult CreateCourseGroup(CourseGroup newCourseGroup);
        OperationResult ReadCourseGroup(string syncId, out CourseGroup readCourseGroup);
        OperationResult UpdateCourseGroup(CourseGroup updatedCourseGroup);
        OperationResult ReplaceCourseGroup(CourseGroup updatedCourseGroup);
        OperationResult DeleteCourseGroup(string syncId);

        BatchOperationResult CreateCourseGroups(ICollection<CourseGroup> newCourseGroups);
        BatchOperationResult ReadCourseGroups(ICollection<string> syncIds, out List<CourseGroup> readCourseGroupList);
        BatchOperationResult UpdateCourseGroups(ICollection<CourseGroup> updatedCourseGroups);
        BatchOperationResult ReplaceCourseGroups(ICollection<CourseGroup> updatedCourseGroups);
        BatchOperationResult DeleteCourseGroups(ICollection<string> syncIds);

        OperationResult ChangeCourseGroupId(string oldId, string newId);
        BatchOperationResult ChangeCourseGroupIds(ICollection<KeyPair> syncIdPairs);

        OperationResult ReadCourseGroupsFromUser(string syncKey, out List<CourseGroup> readCourseGroupList);

        #endregion

        #region Memberships

        OperationResult CreateMembership(string userSyncKey, string hierarchySyncId, HierarchyRole role);
        OperationResult CreateMembership(string userSyncKey, string hierarchySyncId, HierarchyRole role, out string membershipId);
        OperationResult CreateMembership(string membershipId, string userSyncKey, string hierarchySyncId, HierarchyRole role);
        OperationResult UpdateMembership(string userSyncKey, string hierarchySyncId, HierarchyRole role);
        OperationResult UpdateMembership(string membershipId, string userSyncKey, string hierarchySyncId, HierarchyRole role);
        OperationResult DeleteMembership(string userSyncKey, string hierarchySyncId);
        OperationResult DeleteMembership(string membershipId);

        BatchOperationResult CreateMemberships(ICollection<MembershipPair> memberships);
        BatchOperationResult CreateMemberships(ICollection<MembershipPair> memberships, out List<string> membershipIds);
        BatchOperationResult CreateMemberships(IDictionary<string, MembershipPair> memberships);
        BatchOperationResult UpdateMemberships(ICollection<MembershipPair> memberships);
        BatchOperationResult UpdateMemberships(IDictionary<string, MembershipPair> memberships);
        BatchOperationResult DeleteMemberships(ICollection<MembershipPair> memberships);
        BatchOperationResult DeleteMemberships(ICollection<string> membershipIds);


        OperationResult SetHomeOrganisation(string userSyncKey, string hierarchySyncId);
        OperationResult SetHomeOrganisation(string membershipId, string userSyncKey, string hierarchySyncId);

        BatchOperationResult SetHomeOrganisations(ICollection<MembershipPair> memberships);
        BatchOperationResult SetHomeOrganisations(IDictionary<string, MembershipPair> memberships);

        #endregion
    
    }


}
